#!/usr/bin/perl
# ID: /home/.sendmailanalyzer/sa_to_php.pl
#
# Description: 
#
# Sendmail-Analyzer's 'sa_cache' cronjob creates 'cache.pm' files
# in the respective year, month and day directories. These are
# Perl Modules with the condensed statistics of the relevant
# period. In turn Sendmail-Analyzer's 'sa_report.cgi' can include
# them to present the statistics. From within PHP we don't have
# that luxury and can't include the cache.pm files directlry.
# We could parse them, but instead we're doing the neater and
# better way: This script takes command line options about the
# date and timeframe that we want stats for. It also gets told
# for which domain it is (or the whole server). This script then
# polls CCEd if necessary for the aliases of the FQDN in question.
# It then includes the relevant cache.pm files, JSON encodes the
# contained Hashes and dumps them onto the command line.
# From within PHP we just JSON-decode the results and: Voila!
# As neat as it can get.

$DEBUG = 0;

use JSON;
use Getopt::Long;

GetOptions(	'host=s' => \$host, 
			'year=s' => \$year,
           	'week=s' => \$week,
           	'month=s' => \$month,
           	'day=s' => \$day,
           	'hour=s' => \$hour,
           	'domain=s' => \$domain
           	);

$DEBUG && print STDERR "host=$host year=$year week=$week month=$month day=$day hour=$hour\n";

if (! -d "/home/.sendmailanalyzer/$host") {
    $DEBUG && print STDERR "Incorrect statistics directory!\n";
    exit(1);
}

if (!$year) {
	$DEBUG && print STDERR "You must specify a year!\n";
	exit(1);
}

$date = "$year/";
if ($month) {
	$date .= "$month/";
}
if ($day) {
	$date .= "$day/";
}
if ($hour) {
	$date .= "$hour";
}
if ($week) {
	$date .= "weeks/$week/";
}

$liblocation = "/home/.sendmailanalyzer/$host/$date" . "cache.pm" . $domain;

$DEBUG && print STDERR "Path: $liblocation\n";

if (! -f $liblocation) {
	$DEBUG && print STDERR "Error: $liblocation doesn't exist!\n";	
	exit(1);
}

require $liblocation;

$json = encode_json {
					'topauth' => \%::topauth,
					'topsender' => \%::topsender,
					'toprcpt' => \%::toprcpt,
					'topreject' => \%::topreject,
					'topvirus' => \%::topvirus,
					'topdsn' => \%::topdsn,
					'topmaxrcpt' => \%::topmaxrcpt,
					'topmaxsize' => \%::topmaxsize,
					'topspam' => \%::topspam,
					'topspamdetail' => \%::topspamdetail,
					'toperr' => \%::toperr,
					'toppostgrey' => \%::toppostgrey,
					'auth' => \%::auth,
					'messaging' => \%::messaging,
					'spam' => \%::spam,
					'reject' => \%::reject,
					'postgrey' => \%::postgrey,
					'virus' => \%::virus,
					'err' => \%::err,
					'GLOBAL_STATUS' => \%::GLOBAL_STATUS,
					'delivery' => \%::delivery,
					'dsn' => \%::dsn,
					'auth' => \%::auth,
					'messaging' => \%::messaging,
					'spam' => \%::spam,
					'reject' => \%::reject,
					'postgrey' => \%::postgrey,
					'virus' => \%::virus,
					'dsn' => \%::dsn,
					'err' => \%::err,
					'GLOBAL_STATUS' => \%::GLOBAL_STATUS,
					'delivery' => \%::delivery,
					'topauth' => \%::topauth,
					'topsender' => \%::topsender,
					'toprcpt' => \%::toprcpt,
					'topreject' => \%::topreject,
					'topvirus' => \%::topvirus,
					'topdsn' => \%::topdsn,
					'topspam' => \%::topspam,
					'toperr' => \%::toperr,
					'toppostgrey' => \%::toppostgrey,
					'topspamdetail' => \%::topspamdetail,
					'auth' => \%::auth,
					'messaging' => \%::messaging,
					'spam' => \%::spam,
					'reject' => \%::reject,
					'postgrey' => \%::postgrey,
					'virus' => \%::virus,
					'dsn' => \%::dsn,
					'err' => \%::err,
					'GLOBAL_STATUS' => \%::GLOBAL_STATUS,
					'delivery' => \%::delivery,
					'topauth' => \%::topauth,
					'topsender' => \%::topsender,
					'toprcpt' => \%::toprcpt,
					'topreject' => \%::topreject,
					'topvirus' => \%::topvirus,
					'topdsn' => \%::topdsn,
					'topspam' => \%::topspam,
					'toperr' => \%::toperr,
					'toppostgrey' => \%::toppostgrey,
					'topspamdetail' => \%::topspamdetail,
					'auth' => \%::auth,
					'messaging' => \%::messaging,
					'spam' => \%::spam,
					'reject' => \%::reject,
					'postgrey' => \%::postgrey,
					'virus' => \%::virus,
					'dsn' => \%::dsn,
					'err' => \%::err,
					'GLOBAL_STATUS' => \%::GLOBAL_STATUS,
					'delivery' => \%::delivery,
					'topauth' => \%::topauth,
					'topsender' => \%::topsender,
					'toprcpt' => \%::toprcpt,
					'topreject' => \%::topreject,
					'topvirus' => \%::topvirus,
					'topdsn' => \%::topdsn,
					'topspam' => \%::topspam,
					'toperr' => \%::toperr,
					'toppostgrey' => \%::toppostgrey,
					'topspamdetail' => \%::topspamdetail,
					'auth' => \%::auth,
					'messaging' => \%::messaging,
					'spam' => \%::spam,
					'reject' => \%::reject,
					'virus' => \%::virus,
					'dsn' => \%::dsn,
					'err' => \%::err,
					'GLOBAL_STATUS' => \%::GLOBAL_STATUS,
					'delivery' => \%::delivery,
					'topauth' => \%::topauth,
					'topsender' => \%::topsender,
					'toprcpt' => \%::toprcpt,
					'topreject' => \%::topreject,
					'topvirus' => \%::topvirus,
					'topdsn' => \%::topdsn,
					'topspam' => \%::topspam,
					'toperr' => \%::toperr,
					'topspamdetail' => \%::topspamdetail
				};

print "$json\n";

# 
# Copyright (c) 2014 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2014 Team BlueOnyx, BLUEONYX.IT
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#	 notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#	 notice, this list of conditions and the following disclaimer in 
#	 the documentation and/or other materials provided with the 
#	 distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#	 contributors may be used to endorse or promote products derived 
#	 from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 